import { decodeComponentId } from '@previewjs/api';
import { createChromelessWorkspace } from '@previewjs/chromeless';
import { globby } from 'globby';

async function generateScreenshots({
  page,
  frameworkPlugins,
  filePathPattern,
  cwd = process.cwd(),
  generateScreenshotPath,
  onError,
  onScreenshotGenerated
}) {
  const workspace = await createChromelessWorkspace({
    frameworkPlugins,
    rootDirPath: cwd
  });
  const preview = await workspace.preview.start(page);
  const filePaths = await globby(filePathPattern, {
    gitignore: true,
    ignore: ["**/node_modules/**"],
    cwd,
    followSymbolicLinks: false
  });
  const { components } = await workspace.detectComponents({
    filePaths
  });
  for (const component of components) {
    const { filePath, name } = decodeComponentId(component.componentId);
    try {
      await preview.show(component.componentId);
      const screenshotPath = generateScreenshotPath({ filePath, name });
      await preview.iframe.takeScreenshot(screenshotPath);
      if (onScreenshotGenerated) {
        onScreenshotGenerated({ filePath, name, screenshotPath });
      }
    } catch (e) {
      if (onError) {
        const halt = onError(e, { filePath, name });
        if (halt) {
          return;
        }
      } else {
        throw new Error(
          `Unable to generate screenshot for ${component.componentId}`,
          // https://stackoverflow.com/a/42755876
          // @ts-ignore
          { cause: e }
        );
      }
    }
  }
  await preview.stop();
  await workspace.dispose();
}

export { generateScreenshots };
